"""HTTP server module."""
from http.server import SimpleHTTPRequestHandler, ThreadingHTTPServer
from pathlib import Path


class HTTPServer(ThreadingHTTPServer):
    """Simple usage HTTP server.

    This is same implements of ``http.server`` module exec.
    """

    def __init__(self, port: int, directory: Path):  # noqa: D107
        bind = ("", port)
        super().__init__(bind, SimpleHTTPRequestHandler)
        self.directory = directory

    def finish_request(self, request, client_address):  # noqa: D102
        self.RequestHandlerClass(
            request, client_address, self, directory=self.directory
        )


def run_http_server(port: int, directory: Path):
    """Shor-hand of serve by HTTPServer."""
    with HTTPServer(port, directory) as httpd:
        httpd.serve_forever()



"""For watchdog handlers."""
import threading
import time
from dataclasses import dataclass
from pathlib import Path
from typing import List

from sphinx.cmd.build import main as sphinx_build
from watchdog.events import RegexMatchingEventHandler


@dataclass
class SphinxBuildContext:
    """Management params passing to Sphinx CMD."""

    source_dir: Path
    build_dir: Path
    builder_name: str

    @property
    def argv(self) -> List[str]:
        """Return arguments style."""
        return ["-b", self.builder_name, str(self.source_dir), str(self.build_dir)]


class SphinxSourceEventHandler(RegexMatchingEventHandler):
    """Includes build trigger for Sphinx."""

    def __init__(
        self,
        source_dir: Path,
        build_dir: Path,
        builder_name: str,
        extensions=["rst"],
        case_sensitive: bool = False,
    ):  # noqa: D107
        super_args = {
            "case_sensitive": case_sensitive,
            "ignore_directories": [str(build_dir)],
        }
        super_args["regexes"] = rf"{source_dir}/*.*\.{'|'.join(extensions)}$"
        super().__init__(**super_args)
        self.build_context = SphinxBuildContext(
            source_dir=source_dir, build_dir=build_dir, builder_name=builder_name
        )
        self._can_build = False
        self._can_loop = False
        self._thread = threading.Thread(target=self.watch_build)

    def watch_build(self):
        """Core loop to watch build flag.

        This method is called as thread target.
        """
        while self._can_loop:
            time.sleep(1)
            if not self._can_build:
                continue
            sphinx_build(self.build_context.argv)
            # NOTE: Set value not with reason.
            self._can_build = False

    def start_watch(self):
        """Start loop of ``watch_build`` from outside."""
        self._can_loop = True
        self._thread.start()

    def stop_watch(self):
        """Stop loop of ``watch_build``.

        This method is called by main loop (obeserver works).
        """
        self._can_loop = False
        # NOTE: Set value not with reason. (same to watch_build)
        time.sleep(1)
        self._thread.join()

    def on_any_event(self, event):
        """Set flag only (main build works at ``watch_build``)."""
        self._can_build = True



"""CLI entrypoint of sphinx-watch."""
from pathlib import Path

import click
import sphinx
from watchdog.observers import Observer

from . import __version__
from .handlers import SphinxSourceEventHandler
from .httpd import run_http_server


@click.command()
@click.argument(
    "source_dir",
    type=click.Path(
        exists=True, file_okay=False, dir_okay=True, path_type=Path, resolve_path=True
    ),
)
@click.argument(
    "build_dir",
    type=click.Path(
        exists=True, file_okay=False, dir_okay=True, path_type=Path, resolve_path=True
    ),
)
@click.argument("builder", type=click.STRING)
@click.option(
    "--httpd", is_flag=True, show_default=True, default=False, help="Run HTTP server"
)
@click.option("--port", type=int, default=8000, help="Port number for HTTP server")
def main(source_dir: Path, build_dir: Path, builder: str, httpd: bool, port: int):
    """Entrypoint of package."""
    click.echo(f"sphinx-watch version is {__version__}")
    click.echo(f"Sphinx version is {sphinx.__display_version__}")
    click.echo(f"Env is:\n\t{source_dir=}\n\t{build_dir=}\n\t{builder=}\n")
    handler = SphinxSourceEventHandler(source_dir, build_dir, builder)
    observer = Observer()
    observer.schedule(handler, source_dir, recursive=True)
    observer.start()
    try:
        handler.start_watch()
        if httpd:
            run_http_server(port, build_dir)
        while observer.is_alive():
            observer.join(1)
    finally:
        handler.stop_watch()
        observer.stop()
        observer.join()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['sphinx_watch']

package_data = \
{'': ['*']}

install_requires = \
['Sphinx', 'click', 'watchdog']

entry_points = \
{'console_scripts': ['sphinx-watch = sphinx_watch.cli:main']}

setup_kwargs = {
    'name': 'sphinx-watch',
    'version': '0.1.0',
    'description': 'Helper CLI for Sphinx to realtime build',
    'long_description': '============\nsphinx-watch\n============\n\nThis is CLI tool for Documentation eXperimence with Sphinx.\n\nOverview\n========\n\n``sphinx-watch`` provides these features for user local environment.\n\n* Build document of Sphinx when source files are changed by users\n* HTTP server to publish built documentation using ``http.server``\n\nInstallation\n============\n\nUploaded PyPI\n\n.. code-block::\n\n   pip install sphinx-watch\n\nUsage\n=====\n\nBuild only\n\n.. code-block:: console\n\n   sphinx-watch source build html\n\nRun HTTP server\n\n.. code-block:: console\n\n   sphinx-watch source build html --http\n\nSpecify port by running HTTP server\n\n.. code-block:: console\n\n   sphinx-watch source build html --http --port 8080\n\nDevelopment\n===========\n\nThis uses ``poetry`` and ``pre-commit``\n\n.. code-block:: console\n\n   git clone\n   cd sphinx-watch\n   poetry install\n   pre-commit install\n\nLicense\n=======\n\nApache License 2.0\n',
    'author': 'Kazuya Takei',
    'author_email': 'myself@attakei.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/attakei-lab/sphinx-watch',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
